/*
 * Copyright © 2007  Red Hat, Inc. All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY expressed
 * or implied, including the implied warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA. Any Red Hat trademarks that are incorporated in
 * the source code or documentation are not subject to the GNU General
 * Public License and may only be used or replicated with the express
 * permission of Red Hat, Inc.
 *
 * Red Hat Author(s): Ding-Yi Chen <dchen@redhat.com>
 */

/**
 * SECTION: WritRecogn-Type
 * @short_description: Data types that  frequently used in WritRecogn.
 * @stability: Stable
 *
 * This header file hold frequently used data types, as well as many 
 * convenient aliases.
 */


#ifndef WRITRECOGN_TYPE_H_
#define WRITRECOGN_TYPE_H_
#include <glib.h>
#include <sqlite3.h>
#include "utils.h"

/*============================================================
 * Data types from WritRecogn (Core) "package".
 */

typedef enum  {
    RADICAL_TYPE_RADICAL,
    RADICAL_TYPE_RAWSTROKE,
    RADICAL_TYPE_ABSCHARACTER, 
    RADICAL_TYPE_FULLCHARACTER,
} RadicalType;

typedef struct _RadicalArray RadicalArray;
typedef struct _ProgramAssociate ProgramAssociate;
typedef struct _RawWriting RawWriting;


typedef glong RadicalCode;
typedef GTree CoordTree;
typedef Coordinate2D RawStrokeNode;

typedef struct _writRecognRadical Radical;
typedef struct _writRecognRadical_list RadicalList;
typedef struct _writRecognRawStroke RawStroke;
typedef struct _writRecognAbsCharacter AbsCharacter;
typedef struct _writRecognFullCharacter FullCharacter;

#define RADICAL(obj) WRITRECOGN_RADICAL(obj)
#define RAWSTROKE(obj) WRITRECOGN_RAWSTROKE(obj)
#define ABSCHARACTER(obj) WRITRECOGN_ABSCHARACTER(obj)
#define FULLCHARACTER(obj) WRITRECOGN_FULLCHARACTER(obj)
#define RADICAL_LIST(obj) WRITRECOGN_RADICAL_LIST(obj)

typedef enum{
    STROKE_SEQUENCE,
    WUBI98,
    IM_OTHER,
    IM_NOT_SPECIFIED,
    IM_INVALID, //< Error return code.
} InputMethod;

typedef struct _InputCodeRec{
    InputMethod method;
    gchar *inputCode;
} InputCodeRec;


typedef enum  {
    LANG_UNLABELED=0,
    LANG_NUMBER,
    LANG_SYMBOL,
    LANG_EN,
    LANG_JP,
    LANG_KR,
    LANG_ZH_SIMPLIFIED,
    LANG_ZH_TRADITIONAL,
} SupportedLanguage;

typedef GArray LanguageSet;


typedef enum{
    RECOGNIZER=0,
    CHARACTER_DATA_MANAGER
} MainProgramCmd;

typedef struct _ProgFlags {
    unsigned guiFlag: 1;
    unsigned trainingFlag: 1;
    unsigned characterDataModified: 1;
    unsigned hypothesisModified: 1;
    unsigned canvasModified: 1;
    /**
     * FALSE before gtk_widget_show(mainWindowWidget); TRUE otherwise.
     */
    unsigned initializedFlag: 1; 
} ProgFlags;

typedef enum{
    TASK_RUNNING,
    TASK_CANCELED,
    TASK_FAILED,
    TASK_COMPLETED
} TaskOutput;

#define MAX_UNICHAR_CODE_LENGTH 7
#define MAX_CHAR_CODE_LENGTH MAX_UNICHAR_CODE_LENGTH*10



/*============================================================
 * Data types from algors  "package".
 */

typedef gpointer *Hypothesis;
typedef struct _writRecognCharacter_matcher CharacterMatcher;
typedef struct _writRecognStroke_noise_reducer StrokeNoiseReducer;
typedef struct _writRecognStroke_recognizer StrokeRecognizer;	

typedef struct svm_model SVM_MODEL;
typedef struct svm_parameter SVM_PARAMETER;
typedef struct svm_problem SVM_PROBLEM;
typedef struct svm_node SVM_NODE;

typedef enum {
    NUM_OF_MIDDLE_RAW_STROKE_NODE = 1,
    FAREST_DISTANCE,
    INCLUDED_ANGLE,
    //    DIRECT_LENGTH,
} RAW_STROKE_FEATURES;

typedef GArray SVM_FEATURES;	///< feature-array as input of libSVM.

/*============================================================
 * Data types from fileProcessing  "package".
 */


typedef enum {
    RADICAL_FIELD_RADICAL_CODE,
    RADICAL_FIELD_INPUT_METHOD,
    RADICAL_FIELD_INPUT_CODE,
    RADICAL_FIELD_LANG,
    RADICAL_FIELD_VAR_RADICAL,
} RadicalField;

typedef enum  {
    QUERY_RELATION_EQUAL,
    QUERY_RELATION_NOT_EQUAL,
    QUERY_RELATION_LESS_THAN,
    QUERY_RELATION_LESS_OR_EQUAL,
    QUERY_RELATION_GREATER_THAN,
    QUERY_RELATION_GREATER_OR_EQUAL,
} QueryRelation;

typedef struct RadicalQueryItem_ {
    RadicalField field;
    QueryRelation relation;
    gchar* value;
} RadicalQueryItem;


typedef sqlite3 SQLite;
typedef struct _writRecognCharacter_datafile CharacterDataFile;

typedef struct _ProgressData ProgressData;
typedef struct _TrainingProgressData TrainingProgressData;
typedef TrainingProgressData* (* TrainingProgressFunc) (StrokeRecognizer *recognizer,RadicalList *radicaList);


/*============================================================
 * Data types from UI  "package".
 */

/**
 * Canvas Display mode.
 * 
 * <p> In RECOGNIZING_MODE, canvas shows user-drawn strokes, but not background character. 
 * Save functions are disabled.</p>
 * <p>In TRAINING_MODE, canvas shows strokes from database,background characters are from pagno (installed fonts).
 * Save functions are enabled.</p> 
 * <p>In UNIHAN_MODE, canvas shows strokes from database, background characters are from UniHan. 
 * Save functions are enabled.</p> 
 */
typedef enum{
    RECOGNIZING_MODE,
    TRAINING_MODE,
    UNIHAN_MODE
} DisplayMode;

#endif /* WRITRECOGN_TYPE_H_ */

