/*
 * Copyright © 2008  Red Hat, Inc. All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY expressed
 * or implied, including the implied warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA. Any Red Hat trademarks that are incorporated in
 * the source code or documentation are not subject to the GNU General
 * Public License and may only be used or replicated with the express
 * permission of Red Hat, Inc.
 *
 * Red Hat Author(s): Ding-Yi Chen <dchen@redhat.com>
 */

/**
 * Manipulate Database Table.
 * 
 *
 */
#ifndef CHARACTERDATA_SQLITE_TABLE_H_
#define CHARACTERDATA_SQLITE_TABLE_H_


#include <glib.h>
#include <sqlite3.h>
#include "WritRecogn.h"
#include "fileProcessing.h"


typedef enum {
    RELATIVEBOUNDINGBOX_TABLE_INDEX = 0,
    RELATIVERADICALSEQUENCE_TABLE_INDEX,
    LANG_TABLE_INDEX,
    VARIANT_TABLE_INDEX,
    INPUTCODE_TABLE_INDEX,
    STROKEDATA_TABLE_INDEX,
    WRITRECOGN_TABLE_INDEX,
    TABLE_COUNTS
} TableIndex;


typedef struct FieldProperties_ {
    const gchar *name;
    const gchar *dataType;
} FieldProperties;

typedef struct TableProperties_ {
    const gchar *name;
    const FieldProperties *fields;
    int (*parse_result_callback)
     (void *radicalList, int colNum, char **recStr, char **azColName);
    gchar *(*to_insertCmd) (gpointer obj);
    gchar *(*to_updateCmd) (gpointer obj);
} TableProperties;

#define MAIN_KEY_FIELD "RadicalCode"
typedef unsigned TableFlags;
extern unsigned tableFlagValues[];


#define INTEGER_DATA_TYPE "integer"
#define TEXT_DATA_TYPE "text"

#define MAIN_KEY_FIELD_PROPERITIES {MAIN_KEY_FIELD, INTEGER_DATA_TYPE}

/*=========================
 * Relative bounding box table
 */
#define RELATIVEBOUNDINGBOX_TABLE_FIELDS {\
    MAIN_KEY_FIELD_PROPERITIES, {"BoxSequenceId", INTEGER_DATA_TYPE},\
    {"SubRadicalId", INTEGER_DATA_TYPE}, {"SubRadicalCode", INTEGER_DATA_TYPE}, \
    {"Left", INTEGER_DATA_TYPE},{"Top", INTEGER_DATA_TYPE},\
    {"Right", INTEGER_DATA_TYPE},{"Bottom", INTEGER_DATA_TYPE},\
    NULL\
}

typedef enum {
    RELATIVEBOUNDINGBOX_TABLE_FIELD_ID_RADICALCODE = 0,
    RELATIVEBOUNDINGBOX_TABLE_FIELD_ID_BOXSEQUENCEID,
    RELATIVEBOUNDINGBOX_TABLE_FIELD_ID_SUBRADICALID,
    RELATIVEBOUNDINGBOX_TABLE_FIELD_ID_SUBRADICALCODE,
    RELATIVEBOUNDINGBOX_TABLE_FIELD_ID_LEFT,
    RELATIVEBOUNDINGBOX_TABLE_FIELD_ID_TOP,
    RELATIVEBOUNDINGBOX_TABLE_FIELD_ID_RIGHT,
    RELATIVEBOUNDINGBOX_TABLE_FIELD_ID_BOTTOM
} RELATIVEBOUNDINGBOX_TABLE_FIELD_ID;

extern FieldProperties relativeBoundingBoxTable_fields[];

/*=========================
 * Relative radical sequence table
 */
#define RELATIVERADICALSEQUENCE_TABLE_FIELDS {\
    MAIN_KEY_FIELD_PROPERITIES, {"SeqHorz", TEXT_DATA_TYPE},\
    {"SeqVert", TEXT_DATA_TYPE},\
    NULL\
}

typedef enum {
    RELATIVERADICALSEQUENCE_TABLE_FIELD_ID_RADICALCODE = 0,
    RELATIVERADICALSEQUENCE_TABLE_FIELD_ID_SEQHORZ,
    RELATIVERADICALSEQUENCE_TABLE_FIELD_ID_SEQVERT,
} RELATIVERADICALSEQUENCE_TABLE_FIELD_ID;

extern FieldProperties relativeRadicalSequenceTable_fields[];


/*=========================
 * Lang table
 */

#define LANG_TABLE_FIELDS {\
    MAIN_KEY_FIELD_PROPERITIES,{"Lang", TEXT_DATA_TYPE}, \
    NULL\
}

typedef enum {
    LANG_TABLE_FIELD_ID_RADICALCODE = 0,
    LANG_TABLE_FIELD_ID_LANG
} LANG_TABLE_FIELD_ID;

extern FieldProperties langTable_fields[];

/*=========================
 * Variant table
 */

#define VARIANT_TABLE_FIELDS {\
    MAIN_KEY_FIELD_PROPERITIES,{"VarRadicalCode", INTEGER_DATA_TYPE}, \
    NULL\
}

typedef enum {
    VARIANT_TABLE_FIELD_ID_RADICALCODE = 0,
    VARIANT_TABLE_FIELD_ID_VARRADICALCODE
} VARIANT_TABLE_FIELD_ID;

extern FieldProperties variantTable_fields[];

/*=========================
 * InputCode table
 */

#define INPUTCODE_TABLE_FIELDS {\
    MAIN_KEY_FIELD_PROPERITIES,{"Method", INTEGER_DATA_TYPE}, {"InputCode", TEXT_DATA_TYPE},\
    NULL\
}

typedef enum {
    INPUTCODE_TABLE_FIELD_ID_RADICALCODE = 0,
    INPUTCODE_TABLE_FIELD_ID_METHOD,
    INPUTCODE_TABLE_FIELD_ID_INPUTCODE,
} INPUTCODE_TABLE_FIELD_ID;

extern FieldProperties inputCodeTable_fields[];

/*=========================
 * Stroke data table
 */
#define STROKEDATA_TABLE_FIELDS {\
    MAIN_KEY_FIELD_PROPERITIES,{"WritingId", INTEGER_DATA_TYPE},{"StrokeId", INTEGER_DATA_TYPE}, \
    {"NodeId", INTEGER_DATA_TYPE},{"NodeX", INTEGER_DATA_TYPE},{"NodeY", INTEGER_DATA_TYPE}, NULL\
}

typedef enum {
    STROKE_DATA_TABLE_FIELD_ID_RADICALCODE = 0,
    STROKE_DATA_TABLE_FIELD_ID_WRITINGID,
    STROKE_DATA_TABLE_FIELD_ID_STROKEID,
    STROKE_DATA_TABLE_FIELD_ID_NODEID,
    STROKE_DATA_TABLE_FIELD_ID_NODEX,
    STROKE_DATA_TABLE_FIELD_ID_NODEY
} STROKE_DATA_TABLE_FIELD_ID;

extern FieldProperties strokeDataTable_fields[];

/*========================
 * WritRecogn properties table
 */

#define WRITRECOGN_TABLE_FIELDS {\
    {"Version", TEXT_DATA_TYPE}, NULL\
}


/**
 * WRITRECOGN_TABLE_FIELD:
 * @WRITRECOGN_TABLE_FIELD_ID_VERSION: ID of Field Version.
 *
 * writRecognTable stores the information about the database itself.
 * Fields:
 *  Version: version of this database. 
 *  Bump version whenever database structure is modified.
 *  Latest version indicate in WRITRECOGN_DB_VERSION
 */
typedef enum {
    WRITRECOGN_TABLE_FIELD_ID_VERSION = 0,
} WRITRECOGN_TABLE_FIELD_ID;

extern FieldProperties writRecognTable_fields[];

typedef struct _WritRecognDB_properties {
    gchar* version;
} WritRecognDB_properties;

extern WritRecognDB_properties writRecognDB_properties;

extern TableProperties dataTables[];

/**
 * sqliteTable_append_result_radicalList:
 * Append the results from SQL command to the given radicalList.
 * @db: The character database.
 * @table: the table to process.
 * @sqlClause: the SQL clause.
 * @radList: the non-NULL radList which store the result.
 *
 * This function is faster than sqliteTable_append_result_radicalArray(),
 * so unless the full character details are required immediately, 
 * this function is preferred. 
 *
 * @returns: Return radList, NULL when the function failed.
 */
RadicalList *sqliteTable_append_result_radicalList
(sqlite3 * db, TableProperties * table, const gchar * sqlClause, RadicalList * radList);

/**
 * sqliteTable_append_result_radicalArray:
 * Append the results from SQL command to the given radicalArray.
 * @db: The character database.
 * @table: the table to process.
 * @sqlClause: the SQL clause.
 * @radArray: the non-NULL radArray which store the result.
 *
 * This function is slower than sqliteTable_append_result_radicalList(),
 * so unless the full character details are required immediately, 
 * use sqliteTable_append_result_radicalList(). 
 *
 * @returns: Return radArray, NULL when the function failed.
 */
RadicalArray *sqliteTable_append_result_radicalArray
(SQLite * db, TableProperties *table, const gchar *sqlClause, RadicalArray *radArray);

/**
 * sqliteCharacterDataFile_query_allTables_radicalList:
 * Get query result in radicalList from all table of the database.
 * @db: The character database.
 * @queryTemplate: Template of query, see the function description. 
 *
 * The query is submitted in the form of template. 
 * The %s in template will be replaced by the name of the tables.
 * For example: "SELECT * FROM %s WHERE RadicalCode=1000;"
 *
 * This function is faster than sqliteCharacterDataFile_query_allTables_radicalArray(),
 * so unless the full character details are required immediately, 
 * this function is preferred. 
 *
 * @returns: Return a radical list, NULL when the function failed.
 */
RadicalList *sqliteCharacterDataFile_query_allTables_radicalList
(SQLite *db, const gchar *queryTemplate);

/**
 * sqliteCharacterDataFile_query_allTables_radicalArray:
 * Get query result in radicalArray from all table of the database.
 * @db: The character database.
 * @queryTemplate: Template of query, see the function description. 
 *
 * The query is submitted in the form of template. 
 * The %s in template will be replaced by the name of the tables.
 * For example: "SELECT * FROM %s WHERE RadicalCode=1000;"
 *
 * This function is slower than sqliteCharacterDataFile_query_allTables_radicalList(),
 * so unless the full character details are required immediately, 
 * use sqliteCharacterDataFile_query_allTables_radicalList(). 
 *
 * @returns: Return a radical array, NULL when the function failed.
 */
RadicalArray *sqliteCharacterDataFile_query_allTables_radicalArray
(SQLite *db, const gchar *queryTemplate);



int writRecognDB_properties_parse_result_callback
(void *db_properties, int colNum, char **recStr, char **azColName);
gchar *writRecognDB_properties_to_insertCmds(gpointer obj);
gchar *writRecognDB_properties_to_updateCmds(gpointer obj);

#endif                          /*CHARACTERDATA_SQLITE_TABLE_H_ */
