/*
 * Copyright © 2007  Red Hat, Inc. All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY expressed
 * or implied, including the implied warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA. Any Red Hat trademarks that are incorporated in
 * the source code or documentation are not subject to the GNU General
 * Public License and may only be used or replicated with the express
 * permission of Red Hat, Inc.
 *
 * Red Hat Author(s): Ding-Yi Chen <dchen@redhat.com>
 */

/**
 * File processing functions with UI.
 *
 * TUI for file processing functions are defined here.
 * Handles:
 * <ol>
 *   <li>Read/Write SQLite Stroke/Radical/Character data</li>
 *   <li>Read/Write XML-represented Stroke/Radical/Character data</li>
 *   <li>Convert from tomoe: #include "characterData-tomoe.h</li>
 *   <li>Merge with Wubi: #include "characterData-wubi.h"</li>
 *   <li>Generate hypothesis</li>
 * </ol>
 *
 */
#ifndef FILE_FUNCTIONS_UI_H_
#define FILE_FUNCTIONS_UI_H_
#include <gtk/gtk.h>
#include "WritRecogn-Types.h"

#define CHARACTER_DATA_NOT_LOADED_MSG "Character data has not been loaded."
#define FAIL_TO_SAVE_HYPOTHESIE_FILE_MSG "Failed to save svm model in file"
#define LABEL_TEMPLATE "<span size=\"larger\">%s</span>"


/**
 * Data structure necessary to show current progress
 * Used in ProgressBar
 */
struct _ProgressData {
    int timer;
    TaskOutput result;
    gboolean restart;
    GtkWidget *window;
    GtkWidget *pbar;
};

struct _TrainingProgressData {
    ProgressData pdata;
    Hypothesis *hypo;
};

/**
 * Create an instance of ProgressData
 * 
 * @return a new instance of ProgressData, free it if the instance is no longer needed.
 */


ProgressData *progressData_new();

void progressData_reset(ProgressData *pdata);

void progressData_free(ProgressData *pData);

TrainingProgressData *trainingProgressData_new();

void trainingProgressData_free(TrainingProgressData *tProgress);


/*===========================================================
 * characterDataFile functions
 */

/**
 * Return the most preferred filename of user character data.
 *
 * The first non-NULL filename from following list is the preferred file name:
 *
 * <ol>
 *   <li>variable characterDataFileName</li>
 *   <li>predefined DEFAULT_CHARACTER_DATA_FILENAME</li>
 * </ol>
 * @return preferred character data filename.
 */gchar *characterDataFile_get_preferredUserFileName();

gchar *characterDataFile_determine_filename_readable(gchar* filenameBuf);

gchar *characterDataFile_determine_filename_writable(gchar* filenameBuf);

/*===========================================================
 * strokeHypothesisFile functions
 */

/**
 * Return the most preferred filename of user hypothesis.
 *
 * The first non-NULL filename from following list is the preferred file name:
 *
 * <ol>
 *   <li>variable strokeHypothesisFileName</li>
 *   <li>predefined DEFAULT_STROKE_HYPOTHESIS_FILENAME</li>
 * </ol>
 * @return preferred hypothesis filename.
 */
gchar *strokeHypothesisFile_get_preferredUserFileName();


gchar *strokeHypothesisFile_determine_filename_readable(gchar* filenameBuf);

gchar *strokeHypothesisFile_determine_filename_writable(gchar* filenameBuf);

/**
 * Start Training Progress in TUI.
 * 
 * @param filename filename to be saved
 * @return FAILED if training failed, CANCELED if canceled by user,
 *        COMPLETED if training successfully completed.
 * 
 */
TrainingProgressData *strokeHypothesisFile_start_training_TUI(StrokeRecognizer *recognizer,RadicalList *radicalList);
#endif                          /*FILE_FUNCTIONS_UI_H_ */
