/*
 * Copyright © 2007  Red Hat, Inc. All rights reserved.
 *
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY expressed
 * or implied, including the implied warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA. Any Red Hat trademarks that are incorporated in
 * the source code or documentation are not subject to the GNU General
 * Public License and may only be used or replicated with the express
 * permission of Red Hat, Inc.
 *
 * Red Hat Author(s): Ding-Yi Chen <dchen@redhat.com>
 */

#include "inputCodeRec.h"
#include "utils.h"

gchar *INPUT_METHOD_STRINGS[] = {
    "StrokeSequence",
    "Wubi98",
    "Other",
    "NotSpecified",
    NULL
};

gchar *inputMethod_to_string(InputMethod method)
{
    if (method>=0 && method<=IM_NOT_SPECIFIED){
       return INPUT_METHOD_STRINGS[method];
    }
    return NULL;
}

InputMethod inputMethod_parse(const gchar * str)
{
    int i;
    for (i = 0; i <= IM_NOT_SPECIFIED; i++) {
	if (strcmp(str, INPUT_METHOD_STRINGS[i]) == 0) {
	    return i;
	}
    }
    return IM_INVALID;
}

InputCodeRec *inputCodeRec_new(InputMethod method, const gchar *inputCode){
    InputCodeRec *iRec=NEW_INSTANCE(InputCodeRec);
    iRec->method=method;
    iRec->inputCode=g_strdup(inputCode);
    return iRec;
}

void inputCodeRec_free(InputCodeRec *iRec){
    g_free(iRec->inputCode);
    g_free(iRec);
}

void inputCodeRec_copy(InputCodeRec * dest, InputCodeRec * src)
{
    dest->method = src->method;
    if (dest->inputCode)
        g_free(dest->inputCode);
    dest->inputCode=g_strdup(src->inputCode);
}

const char *inputCodeRec_to_string(InputCodeRec * self)
{
    GString *strBuf = g_string_new("");
    g_string_printf(strBuf, "Method=%s Code=[%s]",
		    inputMethod_to_string(self->method),
		    self->inputCode);
    return g_string_free(strBuf, FALSE);
}

InputCodeRecList*
inputCodeRecList_new(){
    return g_array_new(FALSE,TRUE,sizeof(InputCodeRec));
}

InputCodeRec*
inputCodeRecList_new_inputCodeRec(InputCodeRecList* self){
    int id=inputCodeRecList_size(self);
    g_array_set_size(self,id+1);
    return inputCodeRecList_index(self,id);
}


guint inputCodeRecList_size(InputCodeRecList * self)
{
    return self->len;
}


gboolean inputCodeRecList_has_inputMethod(InputCodeRecList * self,InputMethod im){
    int i,len=inputCodeRecList_size(self);
    InputCodeRec *iRec;
    for(i=0;i<len;i++){
	iRec=inputCodeRecList_index(self,i);
	if (iRec->method==im)
	    return TRUE;
    }
    return FALSE;
}

gchar* inputCodeRecList_get_inputCode(InputCodeRecList *self,InputMethod im){
    GString *strBuf=g_string_new(NULL);
    int i;
    gboolean touched=FALSE;
    InputCodeRec *iRec;
    for (i=0;i<self->len;i++) {
        iRec=&g_array_index(self,InputCodeRec,i);
        if (iRec->method==im || im==IM_NOT_SPECIFIED) {
            if (touched){
                g_string_append_printf(strBuf," ");
            }else{
                touched=TRUE;
            }
            g_string_append_printf(strBuf,"%s",iRec->inputCode);
        }
    }
    if (touched){
        return g_string_free(strBuf,FALSE);
    }
    g_string_free(strBuf,TRUE);
    return NULL;
}

gchar**
inputCodeRecList_get_inputCode_array(InputCodeRecList *self,InputMethod im){
    gchar *inputCodes=inputCodeRecList_get_inputCode(self,im);
    gchar **ret=g_strsplit(inputCodes," ",0);
    g_free(inputCodes);
    return ret;
}


InputCodeRec *inputCodeRecList_index(InputCodeRecList * self,
						guint i){
    return &g_array_index(self, InputCodeRec, i);
}

void inputCodeRecList_copy(InputCodeRecList *self,InputCodeRecList *src){
    inputCodeRecList_reset(self);
    G_ARRAY_COPY(self,src);
    int i,len=src->len;
    InputCodeRec *iRec=NULL, *iRecSrc=NULL;
    for(i=0;i<len;i++){
	iRecSrc=inputCodeRecList_index(src,i);
	iRec=inputCodeRecList_index(self,i);
	iRec->inputCode=g_strdup(iRecSrc->inputCode);
    }
}

void inputCodeRecList_reset(InputCodeRecList *self){
    guint len=inputCodeRecList_size(self);
    gint i;
    InputCodeRec *iRec=NULL;
    for(i=0;i<len;i++){
        iRec=inputCodeRecList_index(self,i);
        if (iRec->inputCode){
            g_free(iRec->inputCode);
        }
    }
    G_ARRAY_REMOVE_ALL(self);
}

gint 
inputCodeRecList_find_inputCodeRec_index(InputCodeRecList * self,
	InputCodeRec * iRec){
    gint i;
    gint len = inputCodeRecList_size(self);
    InputCodeRec *tmpIRec;
    for (i = 0; i < len; i++) {
	tmpIRec = &g_array_index(self, InputCodeRec, i);
	if (tmpIRec->method == iRec->method
		&& strcmp(tmpIRec->inputCode,
		    iRec->inputCode) == 0) {
	    return i;
	}
    }
    return -1;
}


InputCodeRec *inputCodeRecList_find_inputCodeRec(InputCodeRecList * self,
						 InputCodeRec * iRec){
    gint index=inputCodeRecList_find_inputCodeRec_index(self,iRec);
    if (index<0)
	return NULL;
    return inputCodeRecList_index(self,index);
}

void inputCodeRecList_append(InputCodeRecList * self,InputCodeRec *iRec){
    InputCodeRec *newIRec=inputCodeRecList_new_inputCodeRec(self);
    inputCodeRec_copy(newIRec,iRec);
}

gboolean inputCodeRecList_remove(InputCodeRecList * self,InputCodeRec *iRec){
    gint index=inputCodeRecList_find_inputCodeRec_index(self,iRec);
    if (index<0)
	return FALSE;
    g_array_remove_index (self,index);
    return TRUE;
}

gchar*
inputCodeRecList_free(InputCodeRecList *self,gboolean free_segment){
    return g_array_free(self,free_segment);
}

