/*
 * Copyright © 2007  Red Hat, Inc. All rights reserved.
 *  
 * This copyrighted material is made available to anyone wishing to use,
 * modify, copy, or redistribute it subject to the terms and conditions
 * of the GNU General Public License v.2. This program is distributed in
 * the hope that it will be useful, but WITHOUT ANY WARRANTY expressed
 * or implied, including the implied warranties of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License 
 * for more details. You should have received a copy of the GNU General
 * Public License along with this program; if not, write to the Free
 * Software Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston,
 * MA 02110-1301, USA. Any Red Hat trademarks that are incorporated in
 * the source code or documentation are not subject to the GNU General
 * Public License and may only be used or replicated with the express
 * permission of Red Hat, Inc. 
 * 
 * Red Hat Author(s): Ding-Yi Chen <dchen@redhat.com> 
 */
 
/**
 * Math functions
 */
#ifndef MATH_FUNCTIONS_H_
#define MATH_FUNCTIONS_H_
#include <glib.h>

#define FLOAT_TO_INT(f) (gint) rintf(f)
#define DOUBLE_TO_INT(d) (gint) rint(d)

typedef struct Coordinate2D_ {
    gint x;
    gint y;
} Coordinate2D;

typedef struct Coordinate3D_ {
    gint x;
    gint y;
    gint z;
} Coordinate3D;


typedef Coordinate2D MathVector2D;

typedef Coordinate3D MathVector3D;

typedef struct MathBox2D_ {
    gint left;
    gint right;
    gint top;
    gint bottom;
} MathBox2D;

typedef MathBox2D BoundingBox2D;
typedef BoundingBox2D BoundingBox;

/*===================================================================
 * Coordinate functions
 */
Coordinate2D* 
newCoordinate2D(gint x,gint y);

Coordinate3D* 
newCoordinate3D(gint x,gint y,gint z);

void 
coordinate2D_copy(Coordinate2D *dest,Coordinate2D *src);

void 
coordinate3D_copy(Coordinate3D *dest,Coordinate3D *src);

void
coordinate2D_set(Coordinate2D *coord,gint x,gint y);

void
coordinate3D_set(Coordinate3D *coord,gint x,gint y,gint z);

MathVector2D*
coordinate2D_to_mathVector2D(MathVector2D *vec,Coordinate2D *start,Coordinate2D *end);

MathVector3D*
coordinate3D_to_mathVector3D(MathVector3D *vec,Coordinate3D *start,Coordinate3D *end);

/**
 * Convert contents Of the Coordinate2D to printable string.
 * 
 * @param coord Coordinate2D to be converted
 * @return String that contains contents of Coordinate2D
 */
gchar*
coordinate2D_to_string(Coordinate2D *coord);

/**
 * Convert contents Of the Coordinate3D to printable string.
 * 
 * @param coord Coordinate3D to be converted
 * @return String that contains contents of Coordinate3D
 */
gchar*
coordinate3D_to_string(Coordinate3D *coord);


gdouble 
coordinate2D_cosine(Coordinate2D *coord1, Coordinate2D *angleCoord, Coordinate2D *coord2);

gdouble 
coordinate3D_cosine(Coordinate3D *coord1, Coordinate3D *angleCoord, Coordinate3D *coord2);


Coordinate2D*
coordinate2D_midpoint(Coordinate2D *result, Coordinate2D *c1, Coordinate2D *c2);

Coordinate3D*
coordinate3D_midpoint(Coordinate3D *result, Coordinate3D *c1, Coordinate3D *c2);

gdouble 
triangle2D_area_given_Coordinate2D(Coordinate2D *c1, Coordinate2D *c2, Coordinate2D *c3);

gdouble 
triangle2D_area_given_points(gint p1_x,gint p1_y, gint p2_x, gint p2_y, gint p3_x, gint p3_y) ;


gdouble
distance2D_between_point_and_line(Coordinate2D *p, Coordinate2D *l1, Coordinate2D *l2);

gdouble
distance2D_between_points(Coordinate2D *l1, Coordinate2D *l2);

/*===================================================================
 * MathVector functions
 */
MathVector3D*
mathVector2D_to_mathVector3D(MathVector3D *vec,MathVector2D *src);

MathVector2D*
mathVector2D_plus(MathVector2D *result, MathVector2D *v1, MathVector2D *v2);

MathVector3D*
mathVector3D_plus(MathVector3D *result, MathVector3D *v1, MathVector3D *v2);

MathVector2D*
mathVector2D_minus(MathVector2D *result, MathVector2D *v1, MathVector2D *v2);

MathVector3D*
mathVector3D_minus(MathVector3D *result, MathVector3D *v1, MathVector3D *v2);

MathVector2D*
mathVector2D_scale(MathVector2D *result, MathVector2D *v,gdouble scale);

MathVector3D*
mathVector3D_scale(MathVector3D *result, MathVector3D *v,gdouble scale);

/**
 * Get the projection of a vector on another vector.
 * 
 * The result vector is the project of v2 on v1.
 * @param result the result vector
 * @param v1 a vector
 * @param v2 a vector
 * @return the signed included angle.
 */
MathVector2D*
mathVector2D_project(MathVector2D *result, MathVector2D *v1, MathVector2D *v2);

/**
 * Get the projection of a vector on another vector.
 * 
 * The result vector is the project of v2 on v1.
 * @param result the result vector
 * @param v1 a vector
 * @param v2 a vector
 * @return the signed included angle.
 */
MathVector3D*
mathVector3D_project(MathVector3D *result, MathVector3D *v1, MathVector3D *v2);

/**
 * Determine whether the target vector is counterclockwise from source vector,
 * 
 * @param src the source vector
 * @param target the target vector
 * @return true if the target vector is counterclockwise from source vector, false otherwise.
 */
gboolean
mathVector2D_is_counterclockwise(MathVector2D *src, MathVector2D *target);

gdouble 
mathVector2D_sine(MathVector2D *v1, MathVector2D *v2);

gdouble 
mathVector3D_sine(MathVector3D *v1, MathVector3D *v2);

gdouble 
mathVector2D_cosine(MathVector2D *v1, MathVector2D *v2);

gdouble 
mathVector3D_cosine(MathVector3D *v1, MathVector3D *v2);


/**
 * Get the included angle between two vectors.
 * 
 * Note that the included angle is positive if v2 is counterclockwise from v1,
 * while the included angle is negative if v2 is clockwise from v1,
 * @param v1 the vector.
 * @param v2 the vector.
 * @return the signed included angle.
 */
gdouble 
mathVector2D_includedAngle(MathVector2D *v1, MathVector2D *v2);

gint
mathVector2D_dotProduct(MathVector2D *v1, MathVector2D *v2);

gint
mathVector3D_dotProduct(MathVector3D *v1, MathVector3D *v2);

/**
 * Get the included angle between two vectors.
 * 
 * Unlike mathVector2D_includedAngle, there is no negative included angle in
 * 3-D space.
 * @param v1 the vector.
 * @param v2 the vector.
 * @return the included angle.
 */
gdouble 
mathVector3D_includedAngle(MathVector3D *v1, MathVector3D *v2);

MathVector3D *
vector_crossProduct(MathVector3D *result, MathVector3D *v1, MathVector3D *v2);

MathVector3D *
vectorPoint_crossProduct(MathVector3D *result,Coordinate3D *v1_start, Coordinate3D * v1_end, Coordinate3D *v2_start, Coordinate3D *v2_end) ;


gdouble 
mathVector3D_EuclideanNorm(MathVector3D *vec);

gdouble 
mathVector2D_EuclideanNorm(MathVector2D *vec);


/*===================================================================
 * MathBox functions
 */

void
mathBox2D_set(MathBox2D* box,gint left,gint top, gint right,gint bottom);

void
mathBox2D_copy(MathBox2D* dest,MathBox2D* src);

/**
 * Convert the box to string.
 *
 * The box will be converted as (left,top,right,down)
 * Note: use g_free to free the unused string.
 *
 * @param box box to be converted
 * @return string representation
 */
gchar*
mathBox2D_to_string(MathBox2D* box);

/**
 * Convert bounding box from string.
 *
 * The box will be set from the string (left,top,right,down)
 *
 * @param box bounding box that stores the result
 * @param str source string
 * @return pointer that point to @box
 */
MathBox2D*
mathBox2D_from_string(MathBox2D* box,const gchar *str);

MathBox2D *mathBox2D_union(MathBox2D *resultBox,MathBox2D *box1,MathBox2D *box2);

#endif /*MATH_FUNCTIONS_H_*/
