/*
 * Modified rom  xmlReader (http://xmlsoft.org/)
 * author: Daniel Veillard
 * copy: see Copyright for the status of this software.
 */
/* Copyright (c) 1994 by Daniel Veillard */
/* Copyright (c) 1998 Jakub Jelinek */
/* Copyright (c) 1997-2001 Red Hat, Inc. */

#include "xmlHandler.h"
#include "utils.h"
const char *XmlNodeTypeStrings[]= {
                                      "None",
                                      "Element",
                                      "Attribute",
                                      "Text",
                                      "CDATA",
                                      "EntityRefence",
                                      "Entity",
                                      "ProcessingInstruction",
                                      "Comment",
                                      "Document",
                                      "DocumentType",
                                      "DocumentFragment",
                                      "Notation",
                                      "Whitespace",
                                      "SignificantWhitespace",
                                      "EndElement",
                                      "EndEntity",
                                      "XmlDeclaration"
                                  };



XML_Node_iteration_func XML_Node_parse_callback=NULL;

void XML_Node_set_parse_callback(XML_Node_iteration_func callback){
    XML_Node_parse_callback=callback;
}



void
XML_Node_parse(xmlTextReaderPtr reader,XML_Node *xml_node, void *userdata) {
    if (XML_Node_parse_callback==NULL){
	verboseMsgPrint(VERBOSE_MSG_CRITICAL,"XML_Node_parse_callback not defined!\n" );
	exit(1);
    }
    XML_Node_parse_callback(reader,xml_node,userdata);
}


void XML_Node_dump_info(xmlTextReaderPtr reader,XML_Node *xml_node, void *userdata){
    const xmlChar *name, *value;
    name = xml_node->name;
    if (name == NULL)
	name = BAD_CAST "--";

    value = xml_node->value;

    printf("%d %s %s %s %s",
	    xmlTextReaderDepth(reader),
	    xmlNodeTypeToString(xmlTextReaderNodeType(reader)),
	    name,
	    xmlTextReaderIsEmptyElement(reader)? "Empty": "NonEmpty",
	    xmlTextReaderHasValue(reader)? "value=" :"NoValue");
    if (value == NULL)
	printf("\n");
    else {
	if (xmlStrlen(value) > 40)
	    printf(" %.40s...\n", value);
	else
	    printf(" %s\n", value);
    }

    if (xml_node->attributeCount>0){
	xmlAttrPtr attr=xml_node->attributeList;
	while(attr!=NULL){
	    printf("\tAttribute name=\"%s\"",attr->name); 
	    if (attr->children!=NULL){
		printf("\ttype=%s",attr->children->name);
		printf("\tvalue=");
		printf("\"%s\"",attr->children->content);
	    }
	    printf("\n");
	    attr=attr->next;
	}
    }
}
           
void xmlTextReader_fill_attributeValues(xmlTextReaderPtr reader,XML_Node *xml_node){
    xmlNodePtr xNodePtr;
    xNodePtr=xmlTextReaderCurrentNode(reader);
    xml_node->attributeCount=xmlTextReaderAttributeCount(reader);
    xml_node->attributeList=xNodePtr->properties;
}
        
XML_Node* 
get_XML_NODE(xmlTextReaderPtr reader){
    XML_Node *xml_node;
    xml_node=NEW_INSTANCE(XML_Node);
    if (xml_node==NULL){
        fputs("Cannot allocate memory for XML_Node",stderr);
        exit(1);
    }
    xml_node->nodeType=xmlTextReaderNodeType(reader);
    xml_node->name=xmlTextReaderConstName(reader);
    xml_node->value=xmlTextReaderConstValue(reader);
    xmlTextReader_fill_attributeValues(reader,xml_node);
    return xml_node;
}

int 
parseXML(xmlTextReaderPtr reader,XML_Node_iteration_func parseCallback, void *userdata){
    int ret;
    XML_Node *xml_node=NULL;
    
    XML_Node_set_parse_callback(parseCallback);

    if (reader != NULL) {
        ret = xmlTextReaderRead(reader);
        while (ret == 1) {
            xml_node=get_XML_NODE(reader);
	    XML_Node_parse(reader,xml_node,userdata);
            ret = xmlTextReaderRead(reader);
        }
        return ret;
    }
    return -1;
}


xmlTextReaderPtr
openXML_File(const char *filename) {
    /*
     * Pass some special parsing options to activate DTD attribute defaulting,
     * entities substitution and DTD validation
     */
    return xmlReaderForFile(filename, NULL,
                            XML_PARSE_DTDATTR |  /* default DTD attributes */
                            XML_PARSE_NOENT |    /* substitute entities */
                            XML_PARSE_DTDVALID); /* validate with the DTD */
}

const char*
xmlNodeTypeToString(int nodeType) {
    if (nodeType>=0 || nodeType<XmlNodeType_XmlDeclaration) {
        return XmlNodeTypeStrings[nodeType];
    }
    return "Unknown";
}

#ifdef TEST_PROGRAM
int main(int argc, char **argv) {
    xmlTextReaderPtr reader;
    if (argc != 2)
        return(1);

    /*
     * this initialize the library and check potential ABI mismatches
     * between the version it was compiled for and the actual shared
     * library used.
     */
    LIBXML_TEST_VERSION

    char *filename=argv[1];

    reader=openXML_File(argv[1]);
    int ret;
    if (reader != NULL) {
	ret=parseXML(reader,XML_Node_dump_info,NULL);
	/*
         * Once the document has been fully parsed check the validation results
         */
        if (xmlTextReaderIsValid(reader) != 1) {
            fprintf(stderr, "Document %s does not validate\n", filename);
        }
        xmlFreeTextReader(reader);
        if (ret != 0) {
            fprintf(stderr, "%s : failed to parse\n", filename);
        }
    } else {
        fprintf(stderr, "Unable to open %s\n", filename);
    }


    /*
     * Cleanup function for the XML library.
     */
    xmlCleanupParser();
    /*
     * this is to debug memory for regression tests
     */
    xmlMemoryDump();
    return(0);
}
#endif //TEST_PROGRAM
